@@ -7,6 +7,8 @@ Created on Fri Mar 11 13:36:39 2016
 import matplotlib.pyplot as plt
 import numpy as np
 
+from collections.abc import Iterable
+
 from . import signal as sn
 from .. import helpers as hp
 from .evaluate import Evaluator
@@ -78,7 +80,6 @@ class Tether(Evaluator):
                          resolution_sf=resolution_sf,
                          filter_time_sf=filter_time_sf)
 
-        self.fe_figure = None
 
     def _sections(self, axis=None, direction=None, cycle=None,
                   concat_direction=False, concat_cycle=False, info=False,
@@ -192,116 +193,117 @@ class Tether(Evaluator):
         extrema.sort()
         return extrema
 
-    def stress_release_pairs(self, axis=None, direction=None, i=None,
-                             slices=True, decimate=None, info=False, **kwargs):
+
+    def stress_release_pairs(self, i=None, axis=None, direction=None,
+                             slices=True, decimate=None, reduce_list=False,
+                             **kwargs):
         """
         Calculate start/stop indices (as segments or slices) of stress/release
         cycle pairs of the requested axis and direction.
 
         Parameters
         ----------
-        axis : str or list of str, optional
-            'x', 'y', or ['x', 'y'] (default)
-        direction : str or list of str, optional
-            'left', 'right', or ['left', 'right'] (default)
         i : int, optional
             The index of the stress release pair to be selected and returned
             from all stress release pairs calculated. Depending on `axis` and
             `direction` the total availabe number of stress release pairs may
             vary.
+        axis : str or list of str, optional
+            'x', 'y', or ['x', 'y'] (default)
+        direction : str or list of str, optional
+            'left', 'right', or ['left', 'right'] (default)
         slices : bool, optional
             Set to False to return segments instead of slices. Default is True.
         decimate : int, optional
             Used to set the step attribute of the returned slices. Only
             evaluated, if `slices` is True.
-        info : bool, optional
         **kwargs : dict, optional
             Only used for compatibility purposes, to be able to call method
             with parameters not defined in method definition.
 
         Returns
         -------
-        stresses : 1D np.ndarray of slices or segments
-        releases : 1D np.ndarray of slices or segments
-        stress_infos : 1D np.ndarray of 1D np.ndarrays of type str
-            The str arrays have the form of [axis, direction, cycle]. Only
-            returned, if parameter `info` is True.
-        release_infos : 1D np.ndarray of 1D np.ndarrays of type str
-            The str arrays have the form of [axis, direction, cycle]. Only
-            returned, if parameter `info` is True.
+        dict
+            stress : 1D np.ndarray of slices or segments
+                idx: 1D np.ndarray of slices or segments
+                info: 1D np.ndarray of 1D np.ndarrays of type str
+                    The str arrays have the form of [axis, direction, cycle].
+            release : 1D np.ndarray of slices or segments
+                info: 1D np.ndarray of 1D np.ndarrays of type str
+                    The str arrays have the form of [axis, direction, cycle].
         """
-        if axis is None:
-            axis = ['x', 'y']
-        if direction is None:
-            direction = ['left', 'right']
-        if isinstance(direction, str):
-            direction = [direction]
-        stress_segments, _stress_infos = self.sections(axis=axis,
-                                                       direction=direction,
-                                                       cycle='stress',
-                                                       slices=False,
-                                                       info=True)
-        release_segments, _release_infos = self.sections(axis=axis,
-                                                         direction=direction,
-                                                         cycle='release',
-                                                         slices=False,
-                                                         info=True)
-        extrema = self._extrema(axis=axis)
-        stresses = np.empty((0, 2), dtype=int)
-        releases = np.empty((0, 2), dtype=int)
-        stress_infos = np.empty((0, 3), dtype=str)
-        release_infos = np.empty((0, 3), dtype=str)
+        axis = ['x', 'y'] if axis is None else axis
+        direction = ['left', 'right'] if direction is None else direction
+        direction = [direction] if isinstance(direction, str) else direction
+
+        # Get unpaired stress/release indices and initialize paired
+        # dictionaries
+        pairs = {}
+        _pairs = {}
+        for cycle in ['stress', 'release']:
+            pairs[cycle] = { 'idx': [], 'info': [] }
+            _pairs[cycle] = {}
+            _pairs[cycle]['idx'], _pairs[cycle]['info'] \
+                = self.sections(axis=axis, direction=direction, cycle=cycle,
+                                slices=False, info=True)
 
         # Group all stress/release cycle pairs according to the extrema
         # A stress release pair corresponds to one extremum, only if the stop
         # of the stress and the start of the release segment equal the
         # extremum. However, either one of the stress or release segment can be
         # missing.
+        extrema = self._extrema(axis=axis)
         for ext in extrema:
-            # Find stress whose stop is equal to extremum
-            stress_idx = stress_segments[:, 1] == ext
-            stress = stress_segments[stress_idx]
-            stress_info = _stress_infos[stress_idx]
-            # Find release whose start is equal to extremum
-            release_idx = release_segments[:, 0] == ext
-            release = release_segments[release_idx]
-            release_info = _release_infos[release_idx]
-            # Create stress slice, if stress is empty, but release is valid
+            # Get pair of stress/release whose stop/start is equal to extremum
+            i_str = _pairs['stress']['idx'][:, 1] == ext
+            i_rls = _pairs['release']['idx'][:, 0] == ext
+            stress = _pairs['stress']['idx'][i_str]
+            release = _pairs['release']['idx'][i_rls]
+            stress_info = _pairs['stress']['info'][i_str]
+            release_info = _pairs['release']['info'][i_rls]
+
             if stress.size == 0 and release.size > 0:
+                # Create stress slice, if only release is valid
                 stress = np.array([[ext, ext]])
-            # Create release slice, if release is empty, but stress is valid
             if release.size == 0 and stress.size > 0:
+                # Create release slice, if only stress is valid
                 release = np.array([[ext, ext]])
             if stress.size > 0 and release.size > 0:
-                # p_p = np.r_[stress, release]  # concatenate stress/release
-                # cycle pair
-                # concatenate with other cycles
-                stresses = np.r_[stresses, stress]
-                releases = np.r_[releases, release]
-                stress_infos = np.r_[stress_infos, stress_info]
-                release_infos = np.r_[release_infos, release_info]
+                pairs['stress']['idx'].append(stress)
+                pairs['release']['idx'].append(release)
+                pairs['stress']['info'].append(stress_info)
+                pairs['release']['info'].append(release_info)
 
-        # Convert segments into slices
-        if slices:
-            stresses = sn.idx_segments_to_slices(stresses, decimate=decimate)
-            releases = sn.idx_segments_to_slices(releases, decimate=decimate)
-
-        # Get the maximum number of stress release pairs
-        stop = len(stresses)
-        # Prevent index overflow and allow negative indices
-        if i is not None:
-            if i < 0:
-                i = stop + i
-            i = max(0, i)
-            i = min(i, stop - 1)
-            s = slice(i, i + 1)
+        # Get the maximum number of stress release pairs, prevent index
+        # overflow, and allow for negative indices
+        if i is None:
+            idx = slice(None)
         else:
-            s = slice(0, stop)
+            if isinstance(i, Iterable):
+                idx = i
+            else:
+                stop = len(pairs['stress']['idx'])
+                if i < 0: i = stop + i
+                i = min(max(0, i), stop - 1)
+                idx = [i]
+
+        # Select idx and convert lists into arrays and segments into slices
+        for cycle in ['stress', 'release']:
+            for sort in ['idx', 'info']:
+                pairs[cycle][sort] = np.concatenate(pairs[cycle][sort])[idx]
+            if slices:
+                pairs[cycle]['idx'] = \
+                    sn.idx_segments_to_slices(pairs[cycle]['idx'],
+                                              decimate=decimate)
+
+        # If only one pair, remove list and select only pair
+        if reduce_list and len(pairs['stress']['idx']) == 1:
+            for cycle in ['stress', 'release']:
+                for sort in ['idx', 'info']:
+                    pairs[cycle][sort] = pairs[cycle][sort][0]
+
+        return pairs
 
-        if info:
-            return stresses[s], releases[s], stress_infos[s], release_infos[s]
-        else:
-            return stresses[s], releases[s]
 
     def baseline_idx(self, axis=None, strict=False, extrapolate=False):
         """
@@ -327,8 +329,9 @@ class Tether(Evaluator):
         if strict:
             # Get only stress/release pairs, i.e. only segment pairs, whose
             # release's stop and stress's start index equal an extremum.
-            stresses, releases = self.stress_release_pairs(axis=axis,
-                                                           slices=False)
+            pairs = self.stress_release_pairs(axis=axis, slices=False)
+            stresses = pairs['stress']['idx']
+            releases = pairs['release']['idx']
         else:
             # Get all stress/release segments, even those without a
             # corresponding extremum.
@@ -386,9 +389,12 @@ class Tether(Evaluator):
         t = self.timevector
         for axis, trace in zip('xy', ['positionX', 'positionY']):
             s = self.get_data(traces=trace) * 1e6  # m -> µm
-            rstr, rrls = self.stress_release_pairs(axis=axis,
-                                                   direction='right')
-            lstr, lrls = self.stress_release_pairs(axis=axis, direction='left')
+            r_str_rls = self.stress_release_pairs(axis=axis, direction='right')
+            l_str_rls = self.stress_release_pairs(axis=axis, direction='left')
+            rstr = r_str_rls['stress']['idx']
+            lstr = l_str_rls['stress']['idx']
+            rrls = r_str_rls['release']['idx']
+            lrls = l_str_rls['release']['idx']
 
             ax.plot(t, s, lw=0.1, ms=2, color='k', alpha=1.0)
 
@@ -467,72 +473,35 @@ class Tether(Evaluator):
 
         return fig
 
-    def force_extension_pair(self, i, axis=None, direction=None, decimate=None,
-                             time=False, twoD=False, posmin=10e-9,
-                             fXYZ_factors=None):
+
+    def force_extension_pairs(self, i=None, axis=None, direction=None,
+                              decimate=None, twoD=False, posmin=10e-9,
+                              dXYZ_factors=None, fXYZ_factors=None,
+                              reduce_list=False, return_calibration=False,
+                              return_settings=False):
+        return self.data_pairs(i=i, axis=axis, direction=direction,
+                               decimate=decimate, fe_data=True, twoD=twoD,
+                               posmin=posmin, dXYZ_factors=dXYZ_factors,
+                               fXYZ_factors=fXYZ_factors,
+                               reduce_list=reduce_list,
+                               return_calibration=return_calibration,
+                               return_settings=return_settings)
+
+    def data_pairs(self, i=None, axis=None, direction=None, decimate=None,
+                   fe_data=False, twoD=False, posmin=10e-9, dXYZ_factors=None,
+                   fXYZ_factors=None, reduce_list=False,
+                   return_calibration=False, return_settings=False):
         """
-        Calculate the force extension pair with index `i`.
+        Return a dictionary of force extension values of stress release pairs.
 
         Parameters
         ----------
         i : int
-            Index of the force_extension pair to be returned.
-        axis : str
-            See method `self.stress_release_pairs()`.
-        direction : str
-            See method `self.stress_release_pairs()`.
-        decimate : int
-            See method `self.stress_release_pairs()`.
-        posmin : float
-            The `posmin` is used to decide wether the magnitude of the force
-            has to be corrected with the sign depending on the position. The
-            `posmin` sets the value the position signal has to be deflected at
-            least to be counted as active pulling on the bead. The value should
-            at least be >= 12 times the standard deviation of the unexcited
-            position signal.
-            Smaller values could (depending on the number of datapoints)
-            possibly lead to falsly detected excitation of the signal.
-
-        Returns
-        -------
-        1D numpy.ndarray of type float
-            Extension values of stress cycles in m.
-        1D numpy.ndarray of type float
-            Force values of stress cycles in N.
-        1D numpy.ndarray of type str
-            (str, str, str), containing the axis, direction, and the cycle.
-            Axis can be either 'x' or 'y'. Direction can be 'left', or 'right'.
-            Cycle is 'stress'.
-        1D numpy.ndarray of type float
-            Extension values of release cycles in m.
-        1D numpy.ndarray of type float
-            Force values of release cycles in N.
-        1D numpy.ndarray of type str
-            (str, str, str), containing the axis, direction, and the cycle.
-            Axis can be either 'x' or 'y'. Direction can be 'left', or 'right'.
-            Cycle is 'release'.
-        """
-        fe_pair = self.force_extension_pairs(axis=axis, direction=direction,
-                                             i=i, decimate=decimate, time=time,
-                                             twoD=twoD, posmin=posmin,
-                                             fXYZ_factors=fXYZ_factors)
-        return next(fe_pair)
-
-    def force_extension_pairs(self, axis=None, direction=None, i=None,
-                              decimate=None, time=False, twoD=False,
-                              posmin=10e-9, fXYZ_factors=None):
-        """
-        Return a generator for force extension values of stress release pairs.
-
-        Parameters
-        ----------
-        str_rls_pairs : tuple of 4 numpy.ndarrays, optional
+            Index of the force extension pair to be yielded.
         axis : str, optional
             See method `self.stress_release_pairs()`.
         direction : str, optional
             See method `self.stress_release_pairs()`.
-        i : int
-            Index of the force extension pair to be yielded.
         decimate : int, optional
             See method `seld.stress_release_pairs()`.
         posmin : float
@@ -545,65 +514,89 @@ class Tether(Evaluator):
             Smaller values could (depending on the number of datapoints)
             possibly lead to falsly detected excitation of the signal.
 
-        Yields
+        Returns
         ------
-        1D numpy.ndarray of type float
-            Extension values of stress cycles in m.
-        1D numpy.ndarray of type float
-            Force values of stress cycles in N.
-        1D numpy.ndarray of type str
-            (str, str, str), containing the axis, direction, and the cycle.
-            Axis can be either 'x' or 'y'. Direction can be 'left', or 'right'.
-            Cycle is 'stress'.
-        1D numpy.ndarray of type float
-            Extension values of release cycles in m.
-        1D numpy.ndarray of type float
-            Force values of release cycles in N.
-        1D numpy.ndarray of type str
-            (str, str, str), containing the axis, direction, and the cycle.
-            Axis can be either 'x' or 'y'. Direction can be 'left', or 'right'.
-            Cycle is 'release'.
+        dict
+            1D numpy.ndarray of type float
+                Extension values of stress cycles in m.
+            1D numpy.ndarray of type float
+                Force values of stress cycles in N.
+            1D numpy.ndarray of type str
+                (str, str, str), containing the axis, direction, and the cycle.
+                Axis can be either 'x' or 'y'. Direction can be 'left', or 'right'.
+                Cycle is 'stress'.
+            1D numpy.ndarray of type float
+                Extension values of release cycles in m.
+            1D numpy.ndarray of type float
+                Force values of release cycles in N.
+            1D numpy.ndarray of type str
+                (str, str, str), containing the axis, direction, and the cycle.
+                Axis can be either 'x' or 'y'. Direction can be 'left', or 'right'.
+                Cycle is 'release'.
         """
-        str_rls_pairs = self.stress_release_pairs(axis=axis,
-                                                  direction=direction,
-                                                  i=i,
-                                                  decimate=decimate,
-                                                  slices=True,
-                                                  info=True)
-        strs, rlss, stri, rlsi = str_rls_pairs
+        pairs = self.stress_release_pairs(i=i, axis=axis, direction=direction,
+                                          decimate=decimate, slices=True,
+                                          reduce_list=False)
 
-        # Get the start from the first stress and the stop from the last
-        # release cycle
-        start = strs[0].start
-        stop = rlss[-1].stop
+        # Get time, extension, and force for all stress/release pairs. Set the
+        # start to the first stress and the stop to the last release cycle
+        start = pairs['stress']['idx'][0].start
+        stop = pairs['release']['idx'][-1].stop
         samples = slice(start, stop)
 
-        # Get extension, force, and stress/release pairs
-        e_f = self.force_extension(samples=samples, twoD=twoD, posmin=posmin,
-                                   fXYZ_factors=fXYZ_factors)  # m,N
-        e = e_f[:, 0]
-        f = e_f[:, 1]
+        t = self.timevector[samples]
+        keys = ['psdXYZ', 'positionXYZ']
+        if fe_data:
+            _data = self.force_extension(samples=samples, twoD=twoD,
+                                         posmin=posmin,
+                                         dXYZ_factors=dXYZ_factors,
+                                         fXYZ_factors=fXYZ_factors,
+                                         return_calibration=return_calibration)
+            keys = keys + ['extension', 'force']
+        else:
+            _data = self.raw_data(samples=samples,
+                                  return_calibration=return_calibration)
 
-        # Get the time
-        if time:
-            t = self.timevector[samples]
+        # Separate all data into stress/release pairs
+        data = {}
 
-        # Yield all stress/release extension/force data pairs
-        for st, rl, sti, rli in zip(strs, rlss, stri, rlsi):
-            _st = slice(st.start - start, st.stop - start, st.step)
-            _rl = slice(rl.start - start, rl.stop - start, rl.step)
-            est = e[_st]
-            fst = f[_st]
-            erl = e[_rl]
-            frl = f[_rl]
-            if time:
-                tst = t[_st]
-                trl = t[_rl]
-                yield est, fst, sti, erl, frl, rli, tst, trl
-            else:
-                yield est, fst, sti, erl, frl, rli
+        for cycle in ['stress', 'release']:
+            data[cycle] = {}
+            data[cycle]['time'] = []
+            for key in keys:
+                data[cycle][key] = []
+            idcs = pairs[cycle]['idx']
+            for idx in idcs:
+                i = slice(idx.start - start, idx.stop - start, idx.step)
+                data[cycle]['time'].append(t[i])
+                for key in keys:
+                    data[cycle][key].append(_data[key][i])
 
-    def samples(self, i, cycle=None, axis=None, direction=None, decimate=None):
+        # If only one pair, remove list and select only pair
+        if reduce_list and len(data['stress']['time']) == 1:
+            for cycle in ['stress', 'release']:
+                for key in data[cycle]:
+                    data[cycle][key] = data[cycle][key][0]
+
+        if return_calibration:
+            data['calibration'] = _data['calibration']
+        if return_settings:
+            data['settings'] = {
+                'i': i,
+                'axis': axis,
+                'direction': direction,
+                'decimate': decimate,
+                'twoD': twoD,
+                'posmin': posmin,
+                'dXYZ_factors': dXYZ_factors,
+                'fXYZ_factors': fXYZ_factors
+            }
+
+        return data
+
+
+    def samples(self, i=None, cycle=None, axis=None, direction=None,
+                decimate=None):
         """
         Get index of samples to be used for functions displacement, force, etc.
 
@@ -612,30 +605,37 @@ class Tether(Evaluator):
         cycle : str or list of str
             cycle can be either 'stress' or 'release'
         """
-        if cycle is None:
-            cycle = ['stress', 'release']
-        if 'stress' in cycle and not 'release' in cycle:  # stress only
-            start = 0
-            stop = 0
-        elif 'release' in cycle and not 'stress' in cycle:  # release only
-            start = 1
-            stop = 1
-        else:  # stress and release
-            start = 0
-            stop = 1
+        cycle = ['stress', 'release'] if cycle is None else cycle
+        start = 'stress'
+        stop = 'release'
+        if 'stress' in cycle and not 'release' in cycle:
+            # stress only
+            stop = start
+        elif 'release' in cycle and not 'stress' in cycle:
+            # release only
+            start = stop
 
-        str_rls_pair = self.stress_release_pairs(axis=axis,
-                                                 direction=direction,
-                                                 i=i,
-                                                 decimate=decimate,
-                                                 slices=False,
-                                                 info=False)
-        idx_start = str_rls_pair[start][0][0]
-        idx_stop = str_rls_pair[stop][0][1]
+        pairs = self.stress_release_pairs(i=i, axis=axis, direction=direction,
+                                          decimate=decimate, slices=True)
+
+        idx_start = pairs[start]['idx'][0].start
+        idx_stop = pairs[stop]['idx'][-1].stop
         idx = slice(idx_start, idx_stop, decimate)
         return idx
 
-    def displacementXYZ(self, samples=None):
+
+    def get_data(self, i=None, cycle=None, axis=None, direction=None,
+                 decimate=None, samples=None, **kwargs):
+        if samples is None and not (i is None and cycle is None
+                                    and axis is None and direction is None
+                                    and decimate is None):
+            samples = self.samples(i=i, cycle=cycle, axis=axis,
+                                   direction=direction, decimate=decimate)
+        data = super().get_data(**kwargs, samples=samples)
+        return data
+
+
+    def displacementXYZ(self, samples=None, dXYZ_factors=None):
         """
         Displacement in m with height dependent calibration factors for X, Y
         and Z.
@@ -646,10 +646,12 @@ class Tether(Evaluator):
         positionZ = data[:, [3]]
         calibration = self.calibration
 
-        dispXYZ = displacementXYZ(calibration, psdXYZ, positionZ)
+        dispXYZ = displacementXYZ(calibration, psdXYZ, positionZ,
+                                  dXYZ_factors=dXYZ_factors)
         return dispXYZ
 
-    def forceXYZ(self, samples=None, fXYZ_factors=None):
+
+    def forceXYZ(self, samples=None, dXYZ_factors=None, fXYZ_factors=None):
         """
         Force in N, that is acting on the tether
         """
@@ -659,10 +661,12 @@ class Tether(Evaluator):
         calibration = self.calibration
 
         fXYZ = forceXYZ(calibration, psdXYZ, positionZ,
-                        fXYZ_factors=fXYZ_factors)
+                        dXYZ_factors=dXYZ_factors, fXYZ_factors=fXYZ_factors)
         return fXYZ
 
-    def force(self, samples=None, twoD=False, posmin=10e-9, fXYZ_factors=None):
+
+    def force(self, samples=None, twoD=False, posmin=10e-9, dXYZ_factors=None,
+              fXYZ_factors=None):
         """
         Magnitude of the force in N acting on the tethered molecule (1D
         numpy.ndarray).
@@ -691,11 +695,12 @@ class Tether(Evaluator):
             psdXYZ[:, Z] = 0.0
 
         fXYZ = forceXYZ(calibration, psdXYZ, positionZ,
-                        fXYZ_factors=fXYZ_factors)
+                        dXYZ_factors=dXYZ_factors, fXYZ_factors=fXYZ_factors)
         f = force(fXYZ, positionXY, posmin=posmin)
         return f
 
-    def distanceXYZ(self, samples=None):
+
+    def distanceXYZ(self, samples=None, dXYZ_factors=None):
         """
         Distance of the attachment point to the bead center as a 3D vector.
         """
@@ -704,10 +709,13 @@ class Tether(Evaluator):
         positionXYZ = data[:, 3:6]
         calibration = self.calibration
 
-        distXYZ = distanceXYZ(calibration, psdXYZ, positionXYZ)
+        distXYZ = distanceXYZ(calibration, psdXYZ, positionXYZ,
+                              dXYZ_factors=dXYZ_factors)
         return distXYZ
 
-    def distance(self, samples=None, twoD=False, posmin=10e-9):
+
+    def distance(self, samples=None, twoD=False, posmin=10e-9,
+                 dXYZ_factors=None):
         """
         Distance of the attachment point to the bead center.
 
@@ -733,10 +741,12 @@ class Tether(Evaluator):
         if twoD:
             psdXYZ[:, Z] = 0.0
 
-        distXYZ = distanceXYZ(calibration, psdXYZ, positionXYZ)
+        distXYZ = distanceXYZ(calibration, psdXYZ, positionXYZ,
+                              dXYZ_factors=dXYZ_factors)
         dist = distance(distXYZ, positionXY, posmin=posmin)
         return dist
 
+
     def extension(self, samples=None, twoD=False, posmin=10e-9):
         """
         Extension of the tethered molecule in m.
@@ -759,8 +769,51 @@ class Tether(Evaluator):
         e = extension(dist, calibration.radius)
         return e
 
-    def force_extension(self, samples=None, twoD=False, posmin=10e-9,
-                        fXYZ_factors=None):
+
+    def raw_data(self, i=None, cycle=None, axis=None, direction=None,
+                 samples=None, return_calibration=False,
+                 return_settings=False):
+        # Get extension and force (in a fast way)
+        data = self.get_data(i=i, cycle=cycle, axis=axis, direction=direction,
+                             traces=['psdXYZ', 'positionXYZ'], samples=samples)
+        psdXYZ = data[:, 0:3]
+        positionXYZ = data[:, 3:6]
+
+        raw_data = {
+            'psdXYZ': psdXYZ,
+            'positionXYZ': positionXYZ
+        }
+        if return_calibration:
+            calibration = self.calibration
+            raw_data['calibration'] = {
+                'beta0': calibration._beta[0],
+                'betam': calibration._beta[1],
+                'kappa0': calibration._kappa[0],
+                'kappam': calibration._kappa[1],
+                'dsurf': calibration.dsurf,
+                'psurf': calibration.psurf,
+                'touchdown': calibration.touchdown,
+                'radius': calibration.radius,
+                'correct_radius': calibration.correct_radius,
+                'corrfactor': calibration.calibsource.corrfactor,
+                'radiusspec': calibration.calibsource.radiusspec,
+                'focalshift': calibration.focalshift
+            }
+        if return_settings:
+            raw_data['settings'] = {
+                'i': i,
+                'cycle': cycle,
+                'axis': axis,
+                'direction': direction,
+                'samples': samples,
+            }
+        return raw_data
+
+
+    def force_extension(self, i=None, cycle=None, axis=None, direction=None,
+                        samples=None, twoD=False, posmin=10e-9,
+                        dXYZ_factors=None, fXYZ_factors=None,
+                        return_calibration=False, return_settings=False):
         """
         Extension (m, first column) of and force (N, second column) acting
         on the tethered molecule (2D numpy.ndarray).
@@ -777,25 +830,47 @@ class Tether(Evaluator):
             Smaller values could (depending on the number of datapoints)
             possibly lead to falsly detected excitation of the signal.
         """
-        # Get extension and force (in a fast way)
-        data = self.get_data(traces=['psdXYZ', 'positionXYZ'], samples=samples)
-        psdXYZ = data[:, 0:3]
-        positionXYZ = data[:, 3:6]
+        data = self.raw_data(i=i, cycle=cycle, axis=axis,
+                             direction=direction, samples=samples,
+                             return_calibration=return_calibration)
+        psdXYZ = data['psdXYZ']
+        positionXYZ = data['positionXYZ']
         positionXY = positionXYZ[:, 0:2]
-        positionZ = data[:, [5]]
+        positionZ = positionXYZ[:, [2]]
         calibration = self.calibration
 
         # 2D or 3D calculation of the distance in Z
         if twoD:
             psdXYZ[:, Z] = 0.0
-        distXYZ = distanceXYZ(calibration, psdXYZ, positionXYZ)
+        displXYZ = displacementXYZ(calibration, psdXYZ, positionZ=positionZ,
+                                   dXYZ_factors=dXYZ_factors)
+        distXYZ = distanceXYZ(calibration, psdXYZ, positionXYZ,
+                              displXYZ=displXYZ, dXYZ_factors=dXYZ_factors)
         dist = distance(distXYZ, positionXY, posmin=posmin)
         e = extension(dist, calibration.radius)
 
         fXYZ = forceXYZ(calibration, psdXYZ, positionZ,
-                        fXYZ_factors=fXYZ_factors)
+                        dXYZ_factors=dXYZ_factors, fXYZ_factors=fXYZ_factors)
         f = force(fXYZ, positionXY, posmin=posmin)
-        return np.c_[e, f]
+
+        data.update({
+            'extension': e,
+            'force': f
+        })
+        if return_settings:
+            data['settings'] = {
+                'i': i,
+                'cycle': cycle,
+                'axis': axis,
+                'direction': direction,
+                'samples': samples,
+                'twoD': twoD,
+                'posmin': posmin,
+                'dXYZ_factors': dXYZ_factors,
+                'fXYZ_factors': fXYZ_factors
+            }
+        return data
+
 
     def info(self, i=0):
         print_info(self, i=i)
@@ -879,52 +954,63 @@ XYZ = hp.slicify([X, Y, Z])
 
 
 def print_info(tether, i=0):
-    stress_release_pair = tether.stress_release_pairs(i=i, info=True)
-    stress, release, stress_info, release_info = stress_release_pair
+    pairs = tether.stress_release_pairs(i=i, slices=True)
+    stress = pairs['stress']['idx'][0]
+    release = pairs['release']['idx'][0]
+    stress_info = pairs['stress']['info'][0]
+    release_info = pairs['release']['info'][0]
     resolution = tether.resolution
-    start_stress_t = stress[0].start / resolution
-    stop_stress_t = stress[0].stop / resolution
-    start_release_t = release[0].start / resolution
-    stop_release_t = release[0].stop / resolution
+    start_stress_t = stress.start / resolution
+    stop_stress_t = stress.stop / resolution
+    start_release_t = release.start / resolution
+    stop_release_t = release.stop / resolution
     print("Stress release pair: #{:03d}".format(i))
     print("Focal shift: {:.3f}".format(tether.calibration.focalshift))
     print("Axis: {}".format(stress_info[0][0]))
     print("  stress")
     print("    t:  {:.3f} s".format(stop_stress_t - start_stress_t))
     print("        {:.2f} s - {:.2f} s".format(start_stress_t, stop_stress_t))
-    print("    datapoints: {}".format(stress[0].stop - stress[0].start))
+    print("    datapoints: {}".format(stress.stop - stress.start))
     print("    z0: {:.2f} nm".format(
-        - np.median(tether.get_data('positionZ', stress[0])) * 1e9))
+        - np.median(tether.get_data('positionZ', stress)) * 1e9))
     print("    h0: {:.2f} nm".format(
-        - np.median(tether.get_data('positionZ', stress[0])) * 1e9
+        - np.median(tether.get_data('positionZ', stress)) * 1e9
                                     * tether.calibration.focalshift))
     print("  release")
     print("    t:  {:.3f} s".format(stop_release_t - start_release_t))
     print("        {:.2f} s - {:.2f} s".format(start_release_t,
                                                stop_release_t))
-    print("    datapoints: {}".format(release[0].stop - release[0].start))
+    print("    datapoints: {}".format(release.stop - release.start))
     print("    z0: {:.2f} nm".format(
-        - np.median(tether.get_data('positionZ', release[0])) * 1e9))
+        - np.median(tether.get_data('positionZ', release)) * 1e9))
     print("    h0: {:.2f} nm".format(
-        - np.median(tether.get_data('positionZ', release[0])) * 1e9
+        - np.median(tether.get_data('positionZ', release)) * 1e9
                                     * tether.calibration.focalshift))
 
 
-def displacementXYZ(calibration, psdXYZ, positionZ=0.0):
+def displacementXYZ(calibration, psdXYZ, positionZ=0.0, dXYZ_factors=None):
     """
     Displacement in m with height dependent calibration factors for X, Y
     and Z.
     """
     dispXYZ = calibration.displacement(psdXYZ, positionZ=positionZ)
+
+    # Optionally use correctur factors for the calculated displacement
+    dXYZ_factors = 1 if dXYZ_factors is None else dXYZ_factors
+
+    dispXYZ *= dXYZ_factors
+
     return dispXYZ
 
 
-def forceXYZ(calibration, psdXYZ, positionZ, fXYZ_factors=None):
+def forceXYZ(calibration, psdXYZ, positionZ, dXYZ_factors=None,
+             fXYZ_factors=None):
     """
     Force acting on the bead
     """
-    displacementXYZ = calibration.displacement(psdXYZ, positionZ=positionZ)
-    fXYZ = calibration.force(displacementXYZ, positionZ=positionZ)
+    dispXYZ = displacementXYZ(calibration, psdXYZ, positionZ=positionZ,
+                              dXYZ_factors=dXYZ_factors)
+    fXYZ = calibration.force(dispXYZ, positionZ=positionZ)
 
     # Optionally use correctur factors for the calculated force
     fXYZ_factors = 1 if fXYZ_factors is None else fXYZ_factors
@@ -937,7 +1023,7 @@ def forceXYZ(calibration, psdXYZ, positionZ, fXYZ_factors=None):
     return fXYZ
 
 
-def force(forceXYZ, positionXY, posmin=10e-9):
+def force(forceXYZ, positionXY, posmin=10e-9, sign_axes=None):
     """
     Parameters
     ----------
@@ -975,13 +1061,14 @@ def force(forceXYZ, positionXY, posmin=10e-9):
     force_sq_sum = np.sum(force_sq, axis=1)
     # Calculate a "weighted" sign. Greater forces have greater influence on the
     # final sign
-    sign_force_sum = np.sign(np.sum(force_sq * signF, axis=1))
+    axs = XY if sign_axes is None else sign_axes
+    sign_force_sum = np.sign(np.sum(force_sq[:,axs] * signF[:,axs], axis=1))
     force = np.sqrt(force_sq_sum) * sign_force_sum
     return force
 
 
-def distanceXYZ(calibration, psdXYZ, positionXYZ, radius=None, focalshift=None,
-                clip_Z=True):
+def distanceXYZ(calibration, psdXYZ, positionXYZ, displXYZ=None, radius=None,
+                focalshift=None, clip_Z=True, dXYZ_factors=None):
     """
     Distance of the attachment point to the bead center as a 3D vector.
     positionXYZ, displacementXYZ and radius need to have the same unit.
@@ -1012,17 +1099,17 @@ def distanceXYZ(calibration, psdXYZ, positionXYZ, radius=None, focalshift=None,
         radius = calibration.radius
     positionZ = positionXYZ[:,2]
 
-    displacementXYZ = calibration.displacement(psdXYZ, positionZ=positionZ)
+    if displXYZ is None:
+        displXYZ = displacementXYZ(calibration, psdXYZ, positionZ=positionZ,
+                                   dXYZ_factors=dXYZ_factors)
 
     # distance, point of attachment of DNA
-    # displacement, displacement of bead out of trap center
-    # radius
+    # displacement, displacement of bead out of trap center radius
     distanceXYZ = positionXYZ.copy()
     # distance from attachment point to center of bead
     # attachmentXY - displacementXY
-    distanceXYZ[:, 0:2] -= displacementXYZ[:, 0:2]
-    # Direction of distance vector is from attachment point
-    # to the bead center
+    distanceXYZ[:, 0:2] -= displXYZ[:, 0:2]
+    # Direction of distance vector is from attachment point to the bead center
     distanceXYZ[:, 0:2] *= -1
 
     # If the bead is free (i.e. above the surface with a distance Z > 0), a
@@ -1035,11 +1122,11 @@ def distanceXYZ(calibration, psdXYZ, positionXYZ, radius=None, focalshift=None,
     distanceXYZ[idx_free, 2] = (- positionXYZ[idx_free, 2] * focalshift
                                 + radius
                                 # distanceZ + radius + displacementZ
-                                + displacementXYZ[idx_free, 2])
+                                + displXYZ[idx_free, 2])
     distanceXYZ[idx_touch, 2] = (- positionXYZ[idx_touch, 2]
                                  + radius
                                  # distanceZ + radius + displacementZ
-                                 + displacementXYZ[idx_touch, 2])
+                                 + displXYZ[idx_touch, 2])
 
     if clip_Z:
         distanceXYZ[:, 2] = distanceXYZ[:, 2].clip(min=radius)
@@ -1078,7 +1165,7 @@ def displacement(displacementXYZ, positionXY, posmin=10e-9):
     return - distance(displacementXYZ, positionXY, posmin=posmin)
 
 
-def distance(distanceXYZ, positionXY, posmin=10e-9):
+def distance(distanceXYZ, positionXY, posmin=10e-9, sign_axes=None):
     """
     Calculate the distance of the attachment point to the bead center.
 
@@ -1115,10 +1202,11 @@ def distance(distanceXYZ, positionXY, posmin=10e-9):
 
     # Square the distances and account for the signs
     distance_sq = distanceXYZ**2
+    dist_sq_sum = np.sum(distance_sq, axis=1)
     # Calculate a "weighted" sign. Greater distances have greater influence on
     # the final sign
-    dist_sq_sum = np.sum(distance_sq, axis=1)
-    sign_dist_sum = np.sign(np.sum(distance_sq * signD, axis=1))
+    axs = XYZ if sign_axes is None else sign_axes
+    sign_dist_sum = np.sign(np.sum(distance_sq[:,axs] * signD[:,axs], axis=1))
     return np.sqrt(dist_sq_sum) * sign_dist_sum
 
 
