@@ -6,13 +6,9 @@ Created on Wed Mar  9 17:07:32 2016
 """
 import matplotlib.pyplot as plt
 import numpy as np
-import os
 import time
-from ipywidgets import interact, IntSlider
-from IPython.core.display import Image, display
 from matplotlib.widgets import SpanSelector
 
-from . import helpers as hp
 from . import traces as tc
 
 
@@ -52,6 +48,7 @@ class GRS(object):
         self._onselect_callbacks = []
         self._spanselector = {}
         self._axspan = {}
+        self.select = True
 
     def add_onselect_callback(self, function=None):
         """
@@ -156,23 +153,26 @@ class GRS(object):
 
         # plot traces
         for i, trace in enumerate(traces):
-            ax[trace].plot(timevector[::decimate], data[:, i][::decimate],
+            ax[trace].plot(timevector[::decimate],
+                           data[:, i][::decimate] * tc.factor(trace),
                            tc.color(trace))
             ax[trace].set_ylabel(tc.label(trace))
             ax[trace].ticklabel_format(useOffset=False)
             ax[trace].set_xlim(*xlim)
             ax[trace].grid(True)
-            # indicators, which span is going to be used
-            self._axspan[trace] = ax[trace].axvspan(self._tmin,
-                                                    self._tmax,
-                                                    facecolor='y',
-                                                    alpha=0.1)
-            # selectors for timespan to be used
-            self._spanselector[trace] = SpanSelector(ax[trace],
-                                                     self._set_time_span,
-                                                     'horizontal',
-                                                     useblit=True,
-                                                     minspan=self._minspan)
+
+            if self.select:
+                # indicators, which span is going to be used
+                self._axspan[trace] = ax[trace].axvspan(self._tmin,
+                                                        self._tmax,
+                                                        facecolor='y',
+                                                        alpha=0.1)
+                # selectors for timespan to be used
+                self._spanselector[trace] = SpanSelector(ax[trace],
+                                                         self._set_time_span,
+                                                         'horizontal',
+                                                         useblit=True,
+                                                         minspan=self._minspan)
 
         # Add OK button
         # TODO: improve scaling of OK button size
@@ -221,8 +221,9 @@ class GRS(object):
 
         # plot traces
         for i, trace in enumerate(traces):
-            ax[trace].lines[0].set_data(timevector[::decimate],
-                                        data[:, i][::decimate])
+            ax[trace].lines[0].set_data(
+                timevector[::decimate],
+                data[:, i][::decimate] * tc.factor(trace))
             ax[trace].set_xlim(*xlim)
 
         self.ifigure.canvas.draw()
@@ -270,7 +271,8 @@ class GRS(object):
         self.ifigure = None
 
     def init_ifig(self, timevector, data, samplingrate, traces, tmin=None,
-                  tmax=None, onselect_cb=None, xlim=None, description=None):
+                  tmax=None, onselect_cb=None, xlim=None, description=None,
+                  select=True):
         """
         Create and show a figure based on the given timevector and data that
         lets the user choose a timespan (tmin, tmax). Whenever the user changes
@@ -302,6 +304,8 @@ class GRS(object):
             str describing the purpose of the region to select. Is used as a
             heading for the figure.
         """
+        self.select = select
+
         # nbagg backend needs to have the figure closed and recreated
         # whenever the code of the cell displaying the figure is executed.
         # A simple update of the figure would let it disappear. Even a
@@ -491,45 +495,3 @@ def set_plot_params(plot_params=None):
         }
 
     plt.rcParams.update(rcparams)
-
-
-def browse_images(directory, prefix=None, suffix=None, extension='.png',
-                  sort_key=None):
-    """
-    Display images of a directory with an interactive widget and a slider in a
-    jupyter notebook, in the order sorted to their filename or a given key
-    function.
-
-    Parameters
-    ----------
-    directory : str
-        The directory the images to be displayed are located in.
-    prefix : str
-        Display only the files beginning with `prefix`.
-    suffix : str
-        Display only the files ending with `suffix`.
-    extension : str, optional
-        The extension of the images that should be displayed. Default is
-        '.png'.
-    sort_key : function
-        Function to be applied to every image filename found, before sorting.
-    """
-    images = hp.files(directory, prefix, suffix, extension, sort_key)
-    stop = len(images)
-    if stop < 1:
-        print("No file found with prefix '%s', suffix '%s', and extension '%s'"
-              % (prefix, suffix, extension))
-        return
-
-    def view_image(i):
-        image = images[i]
-        try:
-            display(Image(filename=image))
-            print(image)
-        except:
-            print('No image found!')
-
-    slider = IntSlider(min=0, max=stop-1, step=1, value=0,
-                       description='Image:')
-
-    return interact(view_image, i=slider)
